// ---------------------------------------
// ProjectX RawRead.cpp
//
// This is an implementation of ProjectX
// RawRead interface.
// This adds large file (>2GB) support to
// ProjectX under OS/2.
//
// RBRi 2006
// ---------------------------------------


#include "java_support.h"

#define INCL_DOSFILEMGR   /* File Manager values */
#define INCL_DOSERRORS    /* DOS error values    */
#include <os2.h>


#include <jni.h>
#include "RawRead.h"
#include <stdio.h>
#include <direct.h>


// #define  _PMPRINTF_
#include "include\PMPRINTF.H"


JNIEXPORT jstring JNICALL Java_RawRead_getLibraryVersion(JNIEnv* aJNIEnv, jclass aJclass) {
    return aJNIEnv->NewStringUTF("0.9.0");
}


JNIEXPORT jint JNICALL Java_RawRead_nativeFindOpen(JNIEnv* aJNIEnv, jobject aJobject, jstring aDirectoryName) {
    jboolean tmpIsCopy;
    const char* tmpDirectoryName;
    DIR* tmpDir;

    tmpDirectoryName = aJNIEnv->GetStringUTFChars(aDirectoryName, &tmpIsCopy);
    _Pmpf(("FindOpen '%s'", tmpDirectoryName));

    tmpDir = opendir(tmpDirectoryName);

    if (JNI_TRUE == tmpIsCopy) {
        // _Pmpf(("FindOpen ReleaseStringUTFChars", tmpDirectoryName));
        aJNIEnv->ReleaseStringUTFChars(aDirectoryName, tmpDirectoryName);
    }

    if (NULL == tmpDir) {
        printf("opendir error");
        _Pmpf(("opendir error"));
        return (jint)-1;
    }

    _Pmpf(("FindOpen result '%u'", tmpDir));
    return (jint)tmpDir;
}


JNIEXPORT jstring JNICALL Java_RawRead_nativeFindNextFile(JNIEnv* aJNIEnv, jobject aJobject, jint aFindHandle) {
    struct dirent* tmpDirEntry;

    _Pmpf(("FindNextFile '%u'", aFindHandle));

    tmpDirEntry = readdir((DIR*)aFindHandle);

    if (NULL == tmpDirEntry) {
        return aJNIEnv->NewStringUTF("");
    }

    _Pmpf(("FindNextFile result '%s'", tmpDirEntry->d_name));
    return aJNIEnv->NewStringUTF(tmpDirEntry->d_name);
}


JNIEXPORT jint JNICALL Java_RawRead_nativeFindClose(JNIEnv* aJNIEnv, jobject aJobject, jint aFindHandle) {
    _Pmpf(("FindClose '%u'", aFindHandle));

    closedir((DIR*)aFindHandle);

    return (jint)0;
}


JNIEXPORT jint JNICALL Java_RawRead_nativeOpenFile(JNIEnv* aJNIEnv, jobject aJobject, jstring aFilename) {
    jboolean tmpIsCopy;
    const char* tmpFileName;

    HFILE tmpResult = 0;
    ULONG ulOpenAction = 0;
    APIRET rc = NO_ERROR;
    LONGLONG tmpFileSize;

    tmpFileSize.ulHi = 0;
    tmpFileSize.ulLo = 0;

    tmpFileName = aJNIEnv->GetStringUTFChars(aFilename, &tmpIsCopy);
    _Pmpf(("OpenFile '%s'", tmpFileName));

    rc = DosOpenL(  tmpFileName,
                    &tmpResult,                   // File handle returned
                    &ulOpenAction,                // Action taken by DosOpen
                    tmpFileSize,
                    0L,
                    OPEN_ACTION_FAIL_IF_NEW | OPEN_ACTION_OPEN_IF_EXISTS,  // Open an existing file only
                    OPEN_FLAGS_NOINHERIT
                        | OPEN_FLAGS_NO_LOCALITY
                        | OPEN_ACCESS_READONLY
                        | OPEN_SHARE_DENYWRITE,         //  | | OPEN_SHARE_DENYNONE Read access only
                    0L);                         // Extended attributes (ignored)

    if (rc != NO_ERROR) {
        printf("DosOpenL error: %u\n", rc);
        _Pmpf(("DosOpenL error: %u\n", rc));
        return (jint)-1;
    }


    if (JNI_TRUE == tmpIsCopy) {
        // PmPrintf("OpenFile ReleaseStringUTFChars", tmpFileName);
        aJNIEnv->ReleaseStringUTFChars(aFilename, tmpFileName);
    }

    _Pmpf(("OpenFile result '%u'", tmpResult));
    return (jint)tmpResult;
}


JNIEXPORT jint JNICALL Java_RawRead_nativeReadFile( JNIEnv* aJNIEnv,
                                                    jobject aJobject,
                                                    jint aFilehandle,
                                                    jbyteArray anArray,
                                                    jint anOffsetInBuffer,
                                                    jint aReadlen) {

    jboolean tmpIsCopy;
    jbyte* tmpByteArrayElements;
    unsigned long tmpResult = 0l;
    APIRET rc = NO_ERROR;

    _Pmpf(("ReadFile '%u' : '%u' : '%u'", aFilehandle, anOffsetInBuffer, aReadlen));
    tmpByteArrayElements = aJNIEnv->GetByteArrayElements(anArray, &tmpIsCopy);

    rc = DosRead(   (HFILE)aFilehandle,
                    &tmpByteArrayElements[anOffsetInBuffer],
                    aReadlen,
                    &tmpResult);

    if (rc != NO_ERROR) {
        printf("DosRead error: %u\n", rc);
        _Pmpf(("DosRead error: %u\n", rc));
        return (jint)-1;
    }

    if (tmpIsCopy) {
        // _Pmpf(("ReadFile IS COPY"));
        aJNIEnv->ReleaseByteArrayElements(anArray, (jbyte*)tmpByteArrayElements, 0);
    }

    _Pmpf(("ReadFile result '%u'", tmpResult));
    return (jint)tmpResult;
}


JNIEXPORT jint JNICALL Java_RawRead_nativeCloseFile(JNIEnv* aJNIEnv, jobject aJobject, jint aFilehandle) {
    APIRET rc = NO_ERROR;

    _Pmpf(("CloseFile '%u'", aFilehandle));

    rc = DosClose((HFILE)aFilehandle);

    if (rc != NO_ERROR) {
        printf("DosClose error: %u\n", rc);
        _Pmpf(("DosClose error: %u\n", rc));

        return (jint)-1;
    }

    return (jint)0;
}


JNIEXPORT jlong JNICALL Java_RawRead_nativeSkipBytes(JNIEnv* aJNIEnv, jobject aJobject, jint aFilehandle, jlong aSkipbytes) {
    APIRET rc = NO_ERROR;
    LONGLONG tmpResult;
    LONGLONG tmpSkipBytes;
    jlong tmpJResult;

    JLONG_2_LONGLONG(aSkipbytes, tmpSkipBytes)
    _Pmpf(("SkipBytes (filehandle %u) [%ld]*4294967296 + [%ld] bytes", aFilehandle, tmpSkipBytes.ulHi, tmpSkipBytes.ulLo));


    rc = DosSetFilePtrL(    (HFILE)aFilehandle,
                            tmpSkipBytes,
                            FILE_CURRENT,
                            &tmpResult);

    if (rc != NO_ERROR) {
        printf("DosSetFilePtrL error: %u\n", rc);
        _Pmpf(("DosSetFilePtrL error: %u\n", rc));

        JLONG_MINUSONE(tmpJResult);
        return tmpJResult;
    }

    LONGLONG_2_JLONG(tmpResult, tmpJResult);

    _Pmpf(("SkipBytes"));
    _Pmpf_jlong(tmpJResult);

    return tmpJResult;
}



JNIEXPORT jlong JNICALL Java_RawRead_nativeGetFileSize(JNIEnv* aJNIEnv, jobject aJobject, jint aFileHandle) {
    FILESTATUS3L tmpFileInfo;
    ULONG tmpFileInfoSize = sizeof(FILESTATUS3L);
    APIRET rc = NO_ERROR;
    LONGLONG tmpFileSize;
    jlong tmpJResult;

    _Pmpf(("GetFileSize '%u'", aFileHandle));
    rc = DosQueryFileInfo(  (HFILE)aFileHandle,
                            FIL_STANDARDL,
                            &tmpFileInfo,
                            tmpFileInfoSize);
    if (rc != NO_ERROR) {
        printf("DosQueryFileInfo error: %u\n", rc);
        _Pmpf(("DosQueryFileInfo error: %u\n", rc));

        JLONG_MINUSONE(tmpJResult);
        return tmpJResult;
    }

    tmpFileSize = tmpFileInfo.cbFile;
    LONGLONG_2_JLONG(tmpFileSize, tmpJResult);

    _Pmpf(("GetFileSize"));
    _Pmpf_jlong(tmpJResult);

    return tmpJResult;
}


