// ---------------------------------------
// ProjectX RawRead.java
//
// This is an implementation of ProjectX
// RawRead interface.
// This adds large file (>2GB) support to
// ProjectX under OS/2.
//
// RBRi 2006
// ---------------------------------------


import java.util.ArrayList;
import java.util.HashMap;
import java.io.*;

import net.sourceforge.dvb.projectx.xinput.topfield_raw.RawReadIF;

public class RawRead implements RawReadIF
{
    private static String videoDirectory;
    HashMap fileSizes = new HashMap();
    boolean debug = false;

    // check DLL load status and errors
    public String GetLoadStatus() {
        if (debug) { System.out.println(" * GetLoadStatus"); }
        return "";
    }


    // ask the dll for the version string
    public static native String getLibraryVersion();


    // directory read functions
    public int findOpen(String aDirectoryName) {
        int tmpResult;

        // reset the cache
        fileSizes = new HashMap();

        tmpResult = nativeFindOpen(aDirectoryName);
        return tmpResult;
    }

    public native int nativeFindOpen(String aDirectoryName);




    public String findNextFile(int aDirectoryHandle) {
        String tmpResult;

        tmpResult = nativeFindNextFile(aDirectoryHandle);
        if ( (null == tmpResult) || ("".equals(tmpResult))  ) {
            return "";
        }

        if ( (".".equals(tmpResult)) || ("..".equals(tmpResult))  ) {
           tmpResult = findNextFile(aDirectoryHandle);
        }

        return tmpResult;
    }

    public native String nativeFindNextFile(int aDirectoryHandle);




    public int findClose(int aDirectoryHandle) {
        int tmpResult;

        tmpResult = nativeFindClose(aDirectoryHandle);
        return tmpResult;
    }

    public native int nativeFindClose(int aDirectoryHandle);




    // file read functions
    public int openFile(String aFilename){
        if (debug) { System.out.println(" * openFile " + aFilename); }

        if (    (".".equals(aFilename))
                || ("..".equals(aFilename))
                || (aFilename.endsWith("."))
           ) {
            return -1;
        }

        int tmpHandle = nativeOpenFile(videoDirectory + aFilename);
        return tmpHandle;
    }

    public native int nativeOpenFile(String aFilename);




    public int readFile(int aFileHandle, byte[] aBuffer, int anOffset, int aLength) {
        int tmpResult;

        tmpResult = nativeReadFile(aFileHandle, aBuffer, anOffset, aLength);
        if ( (tmpResult == 0) && (aLength > 0)) {
            tmpResult = -1;
        }

        return tmpResult;
    }

    public native int nativeReadFile(int aFileHandle, byte[] aBuffer, int anOffset, int aLength);




    public int closeFile(int aFileHandle) {
        int tmpResult;

        tmpResult = nativeCloseFile(aFileHandle);
        return tmpResult;
    }

    public native int nativeCloseFile(int aFileHandle);



    public long skipBytes(int aFileHandle, long aSkipBytes) {
        long tmpResult;
        tmpResult = nativeSkipBytes(aFileHandle, aSkipBytes);

        return tmpResult;
    }

    public native long nativeSkipBytes(int aFileHandle, long aSkipBytes);




    public long getFileSize(String aFilename) {
        int tmpFileHandle;
        Long tmpCachedSize;
        long tmpResult = -1;

        if (debug) { System.out.println(" * getFileSize " + aFilename); }

        if (    (".".equals(aFilename))
                || ("..".equals(aFilename))
           ) {
            System.out.println("getFileSize return -1");
            return -1;
        }

        tmpCachedSize = (Long)fileSizes.get(aFilename);
        if (null != tmpCachedSize) {
            return tmpCachedSize.longValue();
        }

        tmpFileHandle = openFile(aFilename);
        if (-1 < tmpFileHandle) {
            try {
                tmpResult = nativeGetFileSize(tmpFileHandle);
            } finally {
                closeFile(tmpFileHandle);
            }
        }

        fileSizes.put(aFilename, new Long(tmpResult));
        return tmpResult;
    }

    public native long nativeGetFileSize(int aFileHandle);



    public long lastModified(String aFileName) {
        if (debug) { System.out.println(" * lastModified " + aFileName); }
        // TODO
        return System.currentTimeMillis();
    }


    public void add_native_files(ArrayList anArraylist) {
        if (debug) { System.out.println(" * add_native_files"); }

        String tmpFileName;
        int tmpDirectoryHandle = findOpen(videoDirectory);

        if (0 != tmpDirectoryHandle) {
            tmpFileName = findNextFile(tmpDirectoryHandle);
            while (0 < tmpFileName.length()) {
                anArraylist.add(tmpFileName);

                tmpFileName = findNextFile(tmpDirectoryHandle);
            }

            findClose(tmpDirectoryHandle);
        }
    }


    static boolean DllLoaded;

    public boolean AccessEnabled() {
        return DllLoaded;
    }

    // Loads the file Rawread.dll at run-time
    // don't load if access isn't required
    static {
        DllLoaded=false;

        System.out.println(" *************************************************");
        videoDirectory = System.getProperty("videoDir");
        if (null == videoDirectory) {
            System.out.println(" * System property videoDir not set.");
            System.out.println(" * LargeFile support disabled.");
        } else {
            if ( !videoDirectory.endsWith("\\") ) {
                videoDirectory = videoDirectory + "\\";
            }
            System.out.println(" * videoDir '" + videoDirectory + "'");

            File tmpFile = new File(videoDirectory);
            if (!tmpFile.exists()) {
                System.out.println(" * videoDir not exists.");
                System.out.println(" * LargeFile support disabled.");
            } else {
                if (!tmpFile.isDirectory()) {
                    System.out.println(" * videoDir is no directory.");
                    System.out.println(" * LargeFile support disabled.");
                } else {
                    try {
                        String tmpVMVendor;

                        tmpVMVendor = System.getProperty("java.vm.vendor", "");

                        if (-1 < tmpVMVendor.indexOf("IBM")) {
                            System.out.println(" * loadLibrary 'raw_ibm.dll'");
                            System.loadLibrary("raw_ibm");
                            System.out.println(" * library successfull loaded");
                            System.out.println(" * library version " + getLibraryVersion());
                        } else if (-1 < tmpVMVendor.indexOf("Golden Code")) {
                            System.out.println(" * loadLibrary 'raw_gc.dll'");
                            System.loadLibrary("raw_gc");
                            System.out.println(" * library successfull loaded");
                            System.out.println(" * library version " + getLibraryVersion());
                        } else {
                            System.out.println(" * unsupported VM '" + tmpVMVendor + "'");
                            System.out.println(" * LargeFile support disabled.");
                        }

                        DllLoaded=true;
                    } catch (UnsatisfiedLinkError e) {
                        System.out.println(" * loadLibrary failed");
                        e.printStackTrace();
                    }
                }
            }
        }
        System.out.println(" *************************************************");
    }

    // Constructor
    public RawRead() {
        super();
    }
}
